/*
 *  Copyright (C) 2013 Marcus Hirt
 *                     www.hirt.se
 *
 * This software is free:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright (C) Marcus Hirt, 2013, 2015
 */
package se.hirt.w1.impl;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Helper class representing a DHT sensor. Uses the Adafruit_DHT driver, which
 * must be copied to somewhere on the path.
 * 
 * @author Marcus Hirt
 */
public class DHTSensor {
	private final static float DHT_ERROR_TIMEOUT = -1;
	private final static float DHT_ERROR_CHECKSUM = -2;
	private final static float DHT_ERROR_ARGUMENT = -3;
	private final static float DHT_ERROR_GPIO = -4;
	private final static float DHT_SUCCESS = 0;

	static {
		System.loadLibrary("dht");
	}

	/*
	 * Reads values as a double[3] - 0 is temp, 1 is hygrometer, 2 is error
	 * code.
	 */
	private static native float[] readValues(int type, int pin);

	private final static String TEMP_STR = "Temp =";
	private final static String HUM_STR = "Hum =";
	private final int gpioPin;
	private final DHTType type;
	private float[] lastValues;
	private long lastCheck;
	private ExecutorService executor = Executors.newSingleThreadExecutor();

	public DHTSensor(DHTType type, int gpioPin) {
		this.type = type;
		this.gpioPin = gpioPin;
	}

	public String getID() {
		return String.valueOf(gpioPin);
	}

	public float getHumidity() {
		checkForUpdates();
		if (lastValues == null) {
			return Float.NaN;
		}
		return lastValues[1];
	}

	private synchronized void checkForUpdates() {
		long now = System.currentTimeMillis();
		if (now - lastCheck > 3000) {
			float[] newValues = readValues(type.getCode(), gpioPin);
			
			if (newValues == null || newValues[2] != DHT_SUCCESS) {
				float error = newValues[2];
				if (error == DHT_ERROR_GPIO) {
					System.err.println("Error communicating over GPIO with the sensor. Wrong gpio pin?");
				} else if (error == DHT_ERROR_ARGUMENT) {
					System.err.println("Wrong arguments provided to native library.");					
				} 
				// Quite common that reads fail for checksum or timeouts, so we'll just skip this read and try again.
				return;
			} else {
				lastValues = newValues;
				lastCheck = now;
			}
		}
	}

	public float getTemperature() { 
		checkForUpdates();
		if (lastValues == null) {
			return Float.NaN;
		}
		return lastValues[0];
	}

	public void dispose() {
		executor.shutdownNow();
	}

	/**
	 * @param args
	 * @throws InterruptedException 
	 */
	public static void main(String[] args) throws InterruptedException {
		for (int i = 0; i < 30; i++) {
			float[] vals = readValues(2302, 22);
			System.out.println(String.format(
					"Temp: %f  Humidity: %f  Error code:%f", vals[0], vals[1],
					vals[2]));
			System.out.flush();
			Thread.sleep(500);
		}
	}

}
